from typing import Any, Dict, List, NamedTuple, NewType


Vector = NewType("Vector", List[float])

Filter = NewType("Filter", Dict[str, Any])

FieldTransformer = NewType("FieldTransformer", Dict[str, Any])

Schema = NewType("Schema", Dict[str, str])


class Credentials(NamedTuple):
    project: str
    api_key: str
    region: str
    firebase_uid: str



import json

from copy import deepcopy
from typing import Optional, Union, Dict, Any, List

from ai_dashboard.json_encoder import json_encoder
from ai_dashboard.api import endpoints
from ai_dashboard.tabs import (
    abstract_tab,
    data_report,
    category_view,
)
from ai_dashboard.tabs.chart_view import chart_view
from ai_dashboard.tabs.document_view import document_view


class Dashboard:
    def __init__(
        self,
        endpoints: endpoints.Endpoints,
        dataset_id: str,
        deployable_id: str,
        project_id: str,
        configuration: Optional[Dict[str, Any]] = None,
        private: bool = False,
        _id: Optional[str] = None,
        api_key: Optional[str] = None,
        insert_date_: Optional[str] = None,
        updated_at: Optional[str] = None,
    ):
        self._endpoints = endpoints
        self._dataset_id = dataset_id
        self._deployable_id = deployable_id
        self._project_id = project_id
        self._config = {} if configuration is None else configuration
        self._private = private
        self._id = _id
        self._api_key = api_key
        self._insert_date_ = insert_date_
        self._updated_at = updated_at
        self._tabs = self._init_tabs()

    def _init_tabs(self):
        tabs = {}
        for deployable_id, config in self.config["tabs"].items():
            if config["type"] == "REPORT":
                tab = data_report.DataReport(
                    endpoints=self._endpoints,
                    dataset_id=self._dataset_id,
                    config=config,
                )
            if config["type"] == "CLUSTER_VIEW":
                tab = category_view.CategoryView(
                    endpoints=self._endpoints,
                    dataset_id=self._dataset_id,
                    config=config,
                )
            if config["type"] == "CHART_VIEW":
                tab = chart_view.ChartView(
                    endpoints=self._endpoints,
                    dataset_id=self._dataset_id,
                    config=config,
                )
            if config["type"] == "SEARCH_VIEW":
                tab = document_view.DocumentView(
                    endpoints=self._endpoints,
                    dataset_id=self._dataset_id,
                    config=config,
                )
            tabs[deployable_id] = tab
        return tabs

    def DataReport(
        self,
        title: str,
        name: Optional[str] = None,
        colour: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ):
        return data_report.DataReport(
            endpoints=self._endpoints,
            dataset_id=self._dataset_id,
            title=title,
            name=name,
            colour=colour,
            config=config,
        )

    def CategoryView(
        self,
        title: str,
        name: Optional[str] = None,
        colour: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ):
        return category_view.CategoryView(
            endpoints=self._endpoints,
            dataset_id=self._dataset_id,
            title=title,
            name=name,
            colour=colour,
            config=config,
        )

    def ChartView(
        self,
        title: str,
        name: Optional[str] = None,
        colour: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ):
        return chart_view.ChartView(
            endpoints=self._endpoints,
            dataset_id=self._dataset_id,
            title=title,
            name=name,
            colour=colour,
            config=config,
        )

    def DocumentView(
        self,
        title: str,
        name: Optional[str] = None,
        colour: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ):
        return document_view.DocumentView(
            endpoints=self._endpoints,
            dataset_id=self._dataset_id,
            title=title,
            name=name,
            colour=colour,
            config=config,
        )

    @property
    def config(self):
        return self._config

    @property
    def tabs(self) -> List[abstract_tab.Tab]:
        return self._tabs

    def __getitem__(self, index: Union[str, int]) -> abstract_tab.Tab:
        if isinstance(index, int):
            deployable_id = self.config["tabOrder"][index]
        elif isinstance(index, str):
            deployable_id = index
        else:
            raise ValueError

        tab = self.config["tabs"][deployable_id]
        return tab

    def __setitem__(self, index: Union[str, int], tab: abstract_tab.Tab):
        assert issubclass(type(tab), abstract_tab.Tab)
        deployable_id = tab.deployable_id
        if isinstance(index, int):
            assert index < len(self.config["tabOrder"]), "index must exist"
            self.config["tabs"][deployable_id] = tab
            self.config["tabOrder"][index] = deployable_id
        elif isinstance(index, str):
            try:
                list_ind = self.config["tabOrder"].index(index)
            except IndexError:
                self.config["tabOrder"].append(deployable_id)
            else:
                self.config["tabOrder"][list_ind] = deployable_id

            self.config["tabs"][deployable_id] = tab.json()
            self.config["filters"][deployable_id] = {
                "timeRangeFilter": {
                    "field": "insert_date_",
                    "from": "",
                    "to": "",
                },
                "facetableFields": [],
                "selectedFacetValues": {},
                "advancedFilters": [],
                "searchFilter": {
                    "minimumRelevance": 0.1,
                    "file": "",
                    "vectorModels": {},
                    "noHighlighting": False,
                    "questionAnswer": False,
                    "query": "",
                    "textFieldsToSearch": [],
                    "type": "text",
                    "vectorFieldsToSearch": [],
                    "largeInlineFilters": False,
                },
            }
            self.config["filterToggles"][deployable_id] = True
        else:
            raise ValueError

    def pull(self):
        response = self._endpoints._get_deployable(self._deployable_id)
        for attr, value in response.items():
            if attr == "configuration":
                setattr(self, f"_config", value)
            else:
                setattr(self, f"_{attr}", value)

    def push(self):
        config = deepcopy(self.config)
        config["tabs"] = {id: value.json() for id, value in self.tabs.items()}
        config = json_encoder(config)
        assert json.dumps(config), "self.config must be json serializeable"
        return self._endpoints._update_deployable(
            deployable_id=self._deployable_id,
            dataset_id=self._dataset_id,
            configuration=config,
        )

    def append(self, tab: abstract_tab.Tab) -> None:
        assert isinstance(tab, abstract_tab.Tab)
        deployable_id = tab.deployable_id
        self.config["tabs"][deployable_id] = tab.json()
        self.config["tabOrder"].append(deployable_id)
        self.config["filters"][deployable_id] = {
            "timeRangeFilter": {
                "field": "insert_date_",
                "from": "",
                "to": "",
            },
            "facetableFields": [],
            "selectedFacetValues": {},
            "advancedFilters": [],
            "searchFilter": {
                "minimumRelevance": 0.1,
                "file": "",
                "vectorModels": {},
                "noHighlighting": False,
                "questionAnswer": False,
                "query": "",
                "textFieldsToSearch": [],
                "type": "text",
                "vectorFieldsToSearch": [],
                "largeInlineFilters": False,
            },
        }
        self.config["filterToggles"][deployable_id] = True
        self._tabs[deployable_id] = tab

    def delete_all(self):
        self.config["tabs"] = {}
        self.config["tabOrder"] = []

    def delete(self):
        self._endpoints._delete_deployable(self._deployable_id)

    def set_private(self, private: bool):
        if private:
            self._endpoints._share_dashboard(self._deployable_id)
        else:
            self._endpoints._unshare_dashboard(self._deployable_id)



from ai_dashboard import Client


def test_create_dashboard(test_client: Client):
    config = {
        "dataset_id": "steam",
        "authorizationToken": "b74993644646-4d4c-bd11-68287d0a95bb:M2Q2MzQ4MzgtZmZhMC00ZTA0LTlkMDMtZjAyNzQ5Y2QxOTc0:1e3042:EbeVbWzxXnXmFLri0UOrsvQEwI53",
    }

    explore_client = Client(token=config["authorizationToken"])
    dashboard = explore_client.Dashboard(
        dataset_id=config["dataset_id"], name="New Cluster Dashboard"
    )

    # an instruction of silhouette score
    # dashboard.delete_all()

    tab = dashboard.DataReport(title="Cluster Confidence", name="Cluster Confidence")
    tab.add_aggregation(
        [
            {
                "agg": "category",
                "field": "_cluster_.review_translation_vector_.kmeans-25",
                "name": "category_cluster",
                "aggType": "groupby",
            }
        ],
        [
            {
                "name": "avg_voted_up",
                "field": "voted_up_int",
                "agg": "avg",
                "aggType": "metric",
            }
        ],
    )

    dashboard.append(tab)
    dashboard.push()

    # dashboard = client.Dashboard(
    #     dataset_id="iris-csv",
    #     name="My Dashboard" # Dashboard Name
    # )

    tab = dashboard.CategoryView(title="new Category View")
    tab.set_view(
        primary_field="_cluster_.review_translation_vector_.kmeans-25",
        # secondary_field="language",
    )

    dashboard.append(tab)
    dashboard.push()

    tab = dashboard.DocumentView(title="My Document View")
    tab.set_metric(metric="comment_count")
    dashboard.append(tab)
    dashboard.push()

    explore_client.deployables[-1]

    assert True



from typing import Optional, Dict, Any, List

from ai_dashboard.api import endpoints
from ai_dashboard.tabs import abstract_tab


class DocumentView(abstract_tab.Tab):
    ID = "SEARCH_VIEW"

    BLANK: Dict[str, Any] = {
        "activeFilterGroup": "",
        "color": None,
        "configuration": {},
        "name": "",
        "type": ID,
    }

    def __init__(
        self,
        endpoints: endpoints.Endpoints,
        dataset_id: str,
        title: Optional[str] = None,
        name: Optional[str] = None,
        colour: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ):
        super().__init__()

        self._endpoints = endpoints

        if config is not None:
            self._config = config
        else:
            self.reset()

        self.dataset_id = dataset_id
        self.config["colour"] = colour
        self.config["configuration"]["title"] = self.ID if title is None else title
        self.config["name"] = name

    @classmethod
    def from_metric(
        cls,
        title: str,
        metric: str,
        layout_mode: Optional[str] = None,
        layout_template: Optional[str] = None,
        ids_to_filter: Optional[List[str]] = None,
        sort_by_direction: Optional[str] = None,
        name: Optional[str] = None,
        colour: Optional[str] = None,
    ):
        document_view = cls(name=name, title=title, colour=colour)
        document_view.config = {
            "activeFilterGroup": "",
            "color": colour,
            "configuration": {
                "title": title,
                "configuration": {
                    "sortByMetric": metric,
                    "documentViewConfiguration": {
                        "layoutMode": "grid" if layout_mode is None else layout_mode,
                        "layoutTemplate": "text-analysis"
                        if layout_template is None
                        else layout_template,
                    },
                    "page": 1,
                    "documentIdsToFilterOut": ids_to_filter
                    if ids_to_filter is not None
                    else [],
                    "sortByDirection": "desc"
                    if sort_by_direction is None
                    else sort_by_direction,
                },
            },
            "name": name,
            "type": cls.ID,
        }
        return document_view

    def set_metric(
        self,
        metric: str,
        layout_mode: Optional[str] = None,
        layout_template: Optional[str] = None,
        ids_to_filter: Optional[List[str]] = None,
        sort_by_direction: Optional[str] = None,
    ):
        self.config["configuration"] = {
            **self.config["configuration"],
            "configuration": {
                "sortByMetric": metric,
                "documentViewConfiguration": {
                    "layoutMode": "grid" if layout_mode is None else layout_mode,
                    "layoutTemplate": "text-analysis"
                    if layout_template is None
                    else layout_template,
                },
                "page": 1,
                "documentIdsToFilterOut": ids_to_filter
                if ids_to_filter is not None
                else [],
                "sortByDirection": "desc"
                if sort_by_direction is None
                else sort_by_direction,
            },
        }



from setuptools import find_packages, setup

from ai_dashboard import __version__

requirements = [
    "marko>=1.2.2",
    "requests>=2.0.0",
    "kaleido>=0.1.0",
    "plotly>=5.3.1",
    "pandas>=1.3.5",
]


test_requirements = [
    "pytest",
    "pytest-xdist",
    "pytest-cov",
]


setup(
    name="AI Dashboard",
    version=__version__,
    url="https://tryrelevance.com/",
    author="Relevance AI",
    author_email="dev@tryrelevance.com",
    packages=find_packages(),
    setup_requires=["wheel"],
    install_requires=requirements,
    package_data={
        "": [
            "*.ini",
        ]
    },
    extras_require=dict(
        tests=test_requirements,
    ),
)



from typing import Optional, List, Dict, Any


def metric(
    query: List[Dict[str, Any]],
    title: Optional[str] = None,
    show_frequencies: bool = False,
    sort_direction: str = "Descending",
):
    """
    Example for aggregates
    [
        {
            "agg": "category",
            "field": "_cluster_.desc_all-mpnet-base-v2_vector_.kmeans-8",
            "name": "category _cluster_.desc_all-mpnet-base-v2_vector_.kmeans-8",
            "aggType": "groupby",
        },
        {
            "agg": "avg",
            "field": "_sentiment_.desc.cardiffnlp-twitter-roberta-base-sentiment.overall_sentiment_score",
            "name": "avg desc (Sentiment Score)",
            "aggType": "metric",
        },
    ]
    """
    assert sort_direction in {"Descending", "Ascending"}

    for q in query:
        q["aggType"] = "metric"

    return [
        {
            "type": "appBlock",
            "content": [
                {
                    "type": "metricAggregation",
                    "attrs": {
                        "displayType": "column",
                        "sortDirection": sort_direction,
                        "showFrequencies": show_frequencies,
                        "datasetId": "cat_and_dogs",
                        "aggregates": query,
                        "sortBy": "",
                        "title": "" if title is None else title,
                    },
                }
            ],
        }
    ]
